; The LAML library and programs are written by Kurt Normark, Aalborg University, Denmark.
; Copyright (C) 2003 Kurt Normark, normark@cs.auc.dk.
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


(lib-load "xml-in-laml/xml-in-laml.scm")

(laml-style "simple-xhtml1.0-transitional-validating")
(lib-load "xhtml1.0-convenience.scm")

(lib-load "color.scm")
(lib-load "time.scm")
(lib-load "file-read.scm")

(define (photo-show! ast)
  (photo-show-bridging! ast)
  (end-laml)
)

(define photo-show-dir (string-append laml-dir "styles/xml-in-laml/photo-show/"))
(load (string-append  photo-show-dir "mirror/laml-photo-mirror.scm"))

; ---------------------------------------------------------------------------------------------------

(define current-photo-ast #f)

;; A variable that controls if the tool can generate informative messages. 
;; A number where 0 means no messages, 1 means a minimal amount of messages, and 2 means many messages.
(define photo-show-verbosity-level 2)

;; The directory in which to write the output html files from this tool.
;; Defaults to (startup-directory). 
(define photo-destination-dir (startup-directory))

;; The name of the film in which the picture resides
(define film-identification 
  (if (parent-directory photo-destination-dir)
      (file-name-proper (substring photo-destination-dir 0 (- (string-length photo-destination-dir) 1)))
      "ROOT"))

;;; Default settings.
;;; The default settings apply as defaults for photo-entries.
;;; Individual photo entries overwrites the defaults. The defaults stem from
;;; the attributes and the contents of the root photo-show construct.

;; A default string for the upper captions of pictures (a string)
;; Assigned in photo-show-bridging!
(define default-upper-caption #f)

;; A default string for the upper font size of pictures (a number between 1 and 6)
;; Assigned in photo-show-bridging!
(define default-upper-size 0)

;; A default string for the lower captions of pictures (a string)
;; Assigned in photo-show-bridging!
(define default-lower-caption #f)

;; A default string for the lower font size of pictures (a number between 1 and 6)
;; Assigned in photo-show-bridging!
(define default-lower-size 0)

;; A default time (a number, in seconds). Show each picture default-time seconds
;; Assigned in photo-show-bridging!
(define default-time #f)

;; The default photo orientation. Used as the default in make-default-photo-entry.
;; Assigned in photo-show-bridging!
(define default-photo-orientation 'landscape)

;; The default photo size. Used as the default in make-default-photo-entry.
;; Assigned in photo-show-bridging!
(define default-photo-size 'auto)

;; The dimension of the photo overview index tables. An integer. 
;; Assigned in photo-show-bridging!
(define tabular-overview-dimension 3)


;; The width of photos in the visual index
(define visual-index-photo-width 200)

;;; Other important variables.

;; The title of the show. 
;; Intended to be redefined after this tool is loaded. 
;; If not redefined, no title is shown.
(define photo-show-title #f)

; The URL prefix of the pictures. Never redefine this variable.
; The default works for pictures in the parent directory of the html directory.
(define picture-url-prefix "../")

;; The path to a designated photo home directory.
;; Relative to the directory hosting the photos.
(define home-path #f)

;; A URL which allows us to exit from the photo index. Like last-url, this URL must be relative.
;; It is interpreted as relative to the directory in which the photos reside.
;; This variable can be redefined after photo.scm (this tool) has been loaded.
;; If not redefined, no home url is presented.
(define home-url #f)

;; The default color of text.
(define default-photo-text-color #f)

;; The default background color.
(define default-photo-background-color #f)

;; The default transition - IE only.
(define default-transition #f)

;; Determines if raw or pretty printed html is being generated by this tool.
;; A symbol.
(define photo-show-html-mode 'raw)

;; Determines if index pages links to stopped or running photo pages.
(define photo-show-index-linking 'stop)

; Software constant. Not settable from laml photo file.
(define default-menu-font-size-string "10pt")

; The width of the photo table window. Used to determine width of n x n photos in an overview overview.
(define photo-table-window-width 970)

; The height of the photo table window. Used to determine the height of n x x photos in an overview presentation.
(define photo-table-window-height 740)


; The width in percentages for auto landscape photos
(define default-landscape-percentage 95)
(define landscape-percentage default-landscape-percentage)

; The height in percentages for auto portait photos
(define default-portrait-percentage 90)
(define portrait-percentage default-portrait-percentage)

; The name of the camera used. This allows special handling of the photos from different camaras.
; Currently, the rotation suffix Canon G3 specific.
(define camera-used #f)

; Canon G3 relevant
(define rotated-suffix-name "_r1")


(define photo-show-credit "")

;;; Contextual photo show state.

(define photo-show-previous-film #f)
(define photo-show-next-film #f)
(define photo-show-before-first #f)
(define photo-show-after-last #f)

; ---------------------------------------------------------------------------------------------------
; CGI program variables - default initialization of

(define photo-dir-name (file-name-proper (startup-directory)))  ; MzScheme specific. Earlier (current-directory)
; Gives "" - remove trailing '/' before using file-name-proper, or find a better way.

; ---------------------------------------------------------------------------------------------------
; Photo ast selectors

(define (photo-file-name-of photo-ast)
  (get-prop 'file (ast-attributes photo-ast)))

; Return the path of photo-ast, of #f if no part is given
(define (photo-path-of photo-ast)
  (defaulted-get-prop 'path (ast-attributes photo-ast) #f))

(define (photo-show-time-of photo-ast)
  (let ((time (defaulted-get-prop 'time (ast-attributes photo-ast) #f)))
    (if time time default-time)))

(define (photo-upper-caption-of photo-ast)
 (let ((upper-caption-ast (ast-subtree photo-ast "upper-caption")))
  (if upper-caption-ast
      (let ((caption (aggregated-ast-cdata-contents upper-caption-ast)))
        (if (blank-string? caption)
            default-upper-caption
            caption))
      default-upper-caption)))

(define (photo-lower-caption-of photo-ast)
 (let ((lower-caption-ast (ast-subtree photo-ast "lower-caption")))
  (if lower-caption-ast
      (let ((caption (aggregated-ast-cdata-contents lower-caption-ast)))
        (if (blank-string? caption)
            default-lower-caption
            caption))
      default-lower-caption)))

(define (photo-upper-size-of photo-ast)
 (let ((upper-caption-ast (ast-subtree photo-ast "upper-caption")))
  (if upper-caption-ast
      (let ((size (as-symbol (defaulted-get-prop 'size (ast-attributes upper-caption-ast) "auto"))))
        (if size 
            size
            default-upper-size))
      default-upper-size)))

(define (photo-lower-size-of photo-ast)
 (let ((lower-caption-ast (ast-subtree photo-ast "lower-caption")))
  (if lower-caption-ast
      (let ((size (as-number-or-false (defaulted-get-prop 'size (ast-attributes lower-caption-ast) #f))))
        (if size 
            size
            default-lower-size))
      default-lower-size)))

; Returns the actual given photo height, or #f if no such height is supplied in the photo-ast
(define (photo-height-of photo-ast)
  (let ((height (as-number-or-false (defaulted-get-prop 'height (ast-attributes photo-ast) #f))))
    (if height
        height
        #f)))

; Returns the actual given photo width, or #f if no such width is supplied in the photo-ast
(define (photo-width-of photo-ast)
  (let ((width (as-number-or-false (defaulted-get-prop 'width (ast-attributes photo-ast) #f))))
    (if width
        width
        #f)))

(define (photo-orientation-of photo-ast)
  (let ((orientation (as-symbol (defaulted-get-prop 'orientation (ast-attributes photo-ast) "landscape"))))
    (if orientation
        orientation
        default-photo-orientation)))

(define (photo-size-of photo-ast)
  (let ((size (as-symbol (defaulted-get-prop 'size (ast-attributes photo-ast) "auto"))))
    (if size
        size
        default-photo-size)))

; ---------------------------------------------------------------------------------------------------
; Page name functions.

; Return the name of the n'th tabular overview page of dimension dim. 
; No path, and no extension is returned.
(define (tabular-page-name dim n)
  (string-append "index" "-" (as-string dim) "-" (as-string n)))

(define (help-file-name) 
  (if (eq? language-preference 'danish) "help-dk" "help-en"))

(define (about-file-name) 
  (if (eq? language-preference 'danish) "about-dk" "about-en"))


; ---------------------------------------------------------------------------------------------------


(define (make-default-photo-entry photo-file)
  (photo 
    'time (as-string default-time)
    'file photo-file
    'size (as-string default-photo-size)
    'orientation (if (rotated-photo-name? photo-file) "portrait" (as-string default-photo-orientation))
    (upper-caption default-upper-caption 'size (as-string default-upper-size))
    (lower-caption default-lower-caption 'size (as-string default-lower-size))))

(define (rotated-photo-name? photo-file-name)
  (if (and camera-used (equal? camera-used "Canon-G3"))
      (let* ((proper-name (file-name-proper photo-file-name))
             (lgt (string-length proper-name))
             (suffix (if (>= lgt 3)
                         (substring proper-name (- lgt 3) lgt)
                         #f)))
        (and suffix (equal? suffix rotated-suffix-name)))
      #f))
       


; this-photo-entry is a photo AST.
; next-photo-entry and prev-photo-entry may be ASTs, or #f if they do not make sense relative to this-photo-entry
(define (make-single-photo-page this-photo-entry next-photo-entry prev-photo-entry n)
 (let ((this-file-name (photo-file-name-of this-photo-entry))
       (this-path (photo-path-of this-photo-entry))
       (this-show-time (photo-show-time-of this-photo-entry))
       (this-upper-caption (photo-upper-caption-of this-photo-entry))
       (this-upper-size (photo-upper-size-of this-photo-entry))
       (this-lower-caption (photo-lower-caption-of this-photo-entry))
       (this-lower-size (photo-lower-size-of this-photo-entry))
       (given-orientation (photo-orientation-of this-photo-entry))
       (given-size (photo-size-of this-photo-entry))
       (given-photo-height (photo-height-of this-photo-entry))
       (given-photo-width (photo-width-of this-photo-entry))
       (next-file-url (if next-photo-entry (picture-name-to-url (photo-file-name-of next-photo-entry)) (next-url-of-this-show))) 
       (prev-file-url (if prev-photo-entry (picture-name-to-url (photo-file-name-of prev-photo-entry)) (previous-url-of-this-show)))
      )

  (if (not (file-exists? (string-append photo-destination-dir (if this-path this-path "") this-file-name)))
      (display-warning 
         (text-choice "Reference til ikke eksisterende foto:" "Reference to non-existing photo:") 
         (string-append (if this-path this-path "") this-file-name)))

  (if (>= photo-show-verbosity-level 2) (display-message this-file-name))

  (write-timed-photo-page
    this-path this-file-name this-show-time next-file-url prev-file-url this-upper-caption this-upper-size this-lower-caption this-lower-size n
    given-size given-orientation given-photo-width given-photo-height)
  (write-stopped-photo-page
    this-path this-file-name this-show-time next-file-url prev-file-url this-upper-caption this-upper-size this-lower-caption this-lower-size n
    given-size given-orientation given-photo-width given-photo-height)
 )
)


(define (next-url-of-this-show)
  (cond ((eq? photo-show-after-last 'none) "")                                                                      ; No link to next - just ends

        ((eq? photo-show-after-last 'stop-page) "end-page.html")                                                    ; End page

        ((eq? photo-show-after-last 'first-picture)                                                                 ; Cyclic
            (first-url-of-photo-ast current-photo-ast ""))

        ((and photo-show-next-film (directory-exists? (string-append photo-destination-dir photo-show-next-film))   ; First picture in next film
              (eq? photo-show-after-last 'next-film))                              
            (first-url-of-photo-ast (get-photo-ast photo-show-next-film) photo-show-next-film ))

        (else (begin
                (if (> photo-show-verbosity-level 0)
                    (display-message "Problems determining final url of show. Linking to End Page."))
                "end-page.html"))))


(define (previous-url-of-this-show)
  (cond ((eq? photo-show-before-first 'none) "")

        ((eq? photo-show-before-first 'last-picture) 
            (last-url-of-photo-ast current-photo-ast ""))

        ((and photo-show-previous-film (directory-exists? (string-append photo-destination-dir photo-show-previous-film))
              (eq? photo-show-before-first 'previous-film))
            (last-url-of-photo-ast (get-photo-ast photo-show-previous-film) photo-show-previous-film))

        (else (begin
                (if (> photo-show-verbosity-level 0)
                    (display-message "Problems determining first url of show. No previous link from first page."))
                ""))))


; Generate a html url for picture-name.
; If the optional parameter stop? is passed and non-#f, a stop url is returned.
; .form (picture-name-to-url picture-name [stop?])
(define (picture-name-to-url picture-name . optional-parameter-list)
 (let ((stop? (optional-parameter 1 optional-parameter-list #f)))
  (string-append (file-name-proper picture-name)
                 (if stop? "-stop" "")
                  "." "html")))

(define (meta-tag-clauses)
   (list
     (list 'name "Generator" 'content "LAML")
   )
)


; Write a photo on a html page and provide for a continuation to next-picture.
; This-picture is a jpg or gif file names with extensions, but without initial path.
; Next-picture-url is an URL of the next picture (html extension).
; The remaining parameters are captions and their font sizes.
(define (write-timed-photo-page this-path this-picture show-time next-picture-url prev-picture-url
                                upper-caption upper-size lower-caption lower-size n size orientation actual-width actual-height) 
 (let* ((this-picture-name (file-name-proper this-picture))
        (next-picture-name (file-name-proper next-picture-url))
        (this-picture-stop-url (string-append this-picture-name "-stop.html"))
        (colors-list (list default-photo-background-color default-photo-text-color default-photo-text-color default-photo-text-color))
        (bg-color (first colors-list))
        (text-color (second colors-list))
        (link-color (third colors-list))
        (vlink-color (fourth colors-list)))
  (write-html (list photo-show-html-mode 'prolog)
   (html 
    (head 
     (title (text-choice "LAML foto fremvisning" "LAML photo show"))

     (cond ((equal? default-transition "blend")
              (meta 'http-equiv "Page-Enter" 'content "blendTrans(Duration=5.0)"))
           (else 
              (meta 'http-equiv "Page-Enter" 'content "blendTrans(Duration=0.0)")))

     (meta 'name "Generator" 'content "LAML")

     ; Time out script:     
     (script "function navigateNext(nextUrl,ms){setTimeout(\"window.location = \" + \"'\" + nextUrl + \"'\",ms);}" 
             'type "text/javascript")

     ; Key control script:
     (script "" 'src "javascript/photoNavigate.js" 'type "text/javascript")

     (link 'href "stylesheets/photo-style.css"  'rel "stylesheet" 'title "photo-style" 'type "text/css")
 
    )
    (body
      'onload 
      (string-append 
       (js-call "photoNavigateRunning" 
                     (map string-it-single 
                           (list this-picture-stop-url))
           ) "; " 
       (js-call "navigateNext" (list (string-it-single next-picture-url) (* (as-number show-time) 1000))))

;      'scroll "no"

      'bgcolor bg-color
      'text text-color
      'link link-color
      'vlink vlink-color

      (show-picture this-path this-picture upper-caption upper-size lower-caption lower-size n photo-list-length #t 
                        next-picture-url #f #f  size orientation actual-width actual-height)))

    (string-append photo-destination-dir "html/" this-picture-name ".html"))))


; A variant of write-timed-photo-page, which does not apply automatic time out.
(define (write-stopped-photo-page this-path this-picture show-time next-picture-url prev-picture-url
                                 upper-caption upper-size lower-caption lower-size n size orientation actual-width actual-height) 
 (let* ((this-picture-name (file-name-proper this-picture))
        (next-picture-name (file-name-proper next-picture-url))
        (next-picture-init-path (file-name-initial-path next-picture-url))
        (next-picture-stop-url (if (empty-string? next-picture-url) 
                                   "" 
                                   (string-append next-picture-init-path next-picture-name "-stop" ".html")))
        (prev-picture-name (file-name-proper prev-picture-url))
        (prev-picture-init-path (file-name-initial-path prev-picture-url))
        (prev-picture-stop-url (if (empty-string? prev-picture-url) 
                                   ""
                                   (string-append prev-picture-init-path prev-picture-name "-stop" ".html")))
        (run-url next-picture-url)
        (colors-list (list default-photo-background-color default-photo-text-color default-photo-text-color default-photo-text-color))
        (bg-color (first colors-list))
        (text-color (second colors-list))
        (link-color (third colors-list))
        (vlink-color (fourth colors-list))
       )
  (write-html (list photo-show-html-mode 'prolog)
   (html 
    (head 
     (title (text-choice "LAML foto fremvisning" "LAML photo show"))
     (script "" 'src (string-append "javascript/photoNavigate.js") 'type "text/javascript")
     (link 'href "stylesheets/photo-style.css"  'rel "stylesheet" 'title "photo-style" 'type "text/css")
     (meta 'name "Generator" 'content "LAML")

    )
    (body 
      'onload (js-call "photoNavigate" 
                  (append
                    (map string-it-single 
                          (list next-picture-stop-url
                                prev-picture-stop-url
                                run-url
                                (tabular-url "" tabular-overview-dimension (+ 1 (quotient (- n 1) (* tabular-overview-dimension tabular-overview-dimension))))
                                "../index.html"  ; x
                                "" ""
                                ))
                    (list (js-string-array '())))
              )
      'bgcolor bg-color
      'text text-color
      'link link-color
      'vlink vlink-color
      (show-picture this-path this-picture upper-caption upper-size lower-caption lower-size n photo-list-length #f 
                        next-picture-url next-picture-stop-url prev-picture-stop-url size orientation actual-width actual-height)
      


    )
   )
   (string-append photo-destination-dir "html/" this-picture-name "-stop" ".html"))))


; Show the page contents in term of a picture. The picture, captions and font sizes are given as parameters.
; picture-path is #f if not path is given; It means that picture is in .. relatative to the html directory.
; if picture-path is non-#f, the picuture is found in ../picture-path relative to the html directory.
; If stop? is true, a stop link is provided for. If not stop?, a run link bringing us to next-url is presented.
; next-stop-url and prev-stop-url are only used is stop? is false.
(define (show-picture picture-path picture-file-name upper-caption upper-size lower-caption lower-size n total-number 
                      stop? next-url next-stop-url prev-stop-url size orientation actual-width actual-height)
 (let ((hs-space (horizontal-space 3))
       (real-picture-url-prefix
         (string-append picture-url-prefix 
                        (if picture-path picture-path "")))
       (as-string-percent (lambda (x) (string-append (as-string x) "%")))
      )
  (center 
    
     (left-middle-right-banner 
        (con 
          (a (text-choice "Indhold" "Index") 'href "../index.html" 'class "menu-item" (tool-tip "x")) hs-space
          (tabular-link "" (+ 1 (quotient (- n 1) (* tabular-overview-dimension tabular-overview-dimension))) "none" "u")  hs-space
          (if stop?
             (a "Stop" 'href (string-append (file-name-proper picture-file-name) "-stop.html") 'class "menu-item")
             (con (a (text-choice "Kr" "Run") 'href next-url 'class "menu-item" (tool-tip "r") )  hs-space
                  (if (not (empty-string? prev-stop-url))
                      (a (text-choice "Forrige" "Previous") 'href prev-stop-url 'class "menu-item" (tool-tip "p"))
                      (text-choice "Forrige" "Previous"))  hs-space
                  (if (not (empty-string? next-stop-url))
                      (a (text-choice "Nste" "Next") 'href next-stop-url 'class "menu-item" (tool-tip "n")) 
                      (text-choice "Nste" "Next"))
             )
          )
        )
 
        (let ((this-file-name-proper (file-name-proper picture-file-name)))
         (span 'class "menu-item"  
               film-identification
               hs-space "-" hs-space 
               (a 'href (string-append "laml-fragments.html" "#" this-file-name-proper ) (as-string n) " : " (as-string total-number) 'css:text-decoration "none") 
               hs-space "-" hs-space 
               (a 'href (string-append real-picture-url-prefix picture-file-name) (font 'size "2" this-file-name-proper) 'css:text-decoration "none")
         )
        )

        (con
         (if home-url
            (a (text-choice "Hjem" "Home") 'href (string-append "../" home-url) 'class "menu-item")
            "") hs-space
         (a (text-choice "Om" "About") 'href (string-append (about-file-name) ".html") 'class "menu-item") hs-space
         (a (text-choice "Hjlp" "Help") 'href (string-append (help-file-name) ".html") 'class "menu-item")
        )
     )

     (if upper-caption 
         (list (font 'size (as-string upper-size) 'color default-photo-text-color (b upper-caption)) (p))
         (vertical-space 1))
  
     (cond ((and (eq? size 'original))  
              (img 'src (string-append real-picture-url-prefix picture-file-name) 'alt ""))

           ((and (eq? size 'auto) (eq? orientation 'landscape) (not actual-width) (not actual-height))  
              (img 'width (as-string-percent landscape-percentage)  'src (string-append real-picture-url-prefix picture-file-name) 'alt ""))

           ((and (eq? size 'auto) (eq? orientation 'portrait) (not actual-width) (not actual-height))
              (img 'height (as-string-percent portrait-percentage) 'src (string-append real-picture-url-prefix picture-file-name) 'alt ""))

           ((and (eq? size 'explicit) actual-height actual-width)
              (img 'height (as-string actual-height) 'width (as-string actual-width) 
                         'src (string-append real-picture-url-prefix picture-file-name) 'alt ""))
           ((and (eq? size 'explicit) actual-height)
              (img 'height (as-string actual-height) 'src (string-append real-picture-url-prefix picture-file-name) 'alt ""))
           ((and actual-width (eq? size 'explicit))
              (img 'width (as-string actual-width) 'src (string-append real-picture-url-prefix picture-file-name) 'alt ""))
           (else (img 'src (string-append real-picture-url-prefix picture-file-name) 'alt "") ))

     (p)

     (if lower-caption 
         (list (font 'size (as-string lower-size) 'color default-photo-text-color (b lower-caption)) (p))
         "")

)))






;;; Top level functions.
;;; The functions in this section are used in the LAML script which generates the photo show.

;; Make the photo-show from photo-list in terms of individual HTML files and an index file.
;; Photo-list is a list of photo-ASTs.
(define (make-photo-show photo-list previous-film-path next-film-path)
 (let ((html-dir (string-append photo-destination-dir "html"))
       (internal-dir (string-append photo-destination-dir "internal"))
      )

  (make-photo-index photo-list previous-film-path next-film-path)


  (map4
    make-single-photo-page
    photo-list
    (if (not (null? photo-list)) (append (cdr photo-list) (list #f)) '())
    (if (not (null? photo-list)) (append (list #f) (butlast photo-list)) '())
    (number-interval 1 (length photo-list))
  )

  ; write the first.lsp file
  ; (if (not (null? photo-list))
  ;  (file-write
  ;   (file-name-proper (photo-file-name-of (first photo-list)))
  ;   (string-append (startup-directory) "first.lsp")))

  (ensure-directory-existence! (string-append photo-destination-dir "html/") "javascript")
  (copy-text-file (string-append photo-show-dir "javascript/" "photoNavigate.js")
                  (string-append photo-destination-dir "html/" "javascript/photoNavigate.js")
                  #t)                  ; overwrite if necessary

  (ensure-directory-existence! (string-append photo-destination-dir "html/") "stylesheets")

  (let* ((style-sheet-string (read-text-file (string-append photo-show-dir "stylesheets/" "photo-style.css")))
         (modified-style-sheet-string
           (replace-string
            (replace-string style-sheet-string "@background-color@" default-photo-background-color)
            "@menu-font-size@"
            default-menu-font-size-string))
         (target-file-path (string-append photo-destination-dir "html/" "stylesheets/photo-style.css"))
        )
    (if (file-exists? target-file-path) (delete-file target-file-path))
    (write-text-file modified-style-sheet-string target-file-path))

  ;(copy-text-file (string-append photo-show-dir "stylesheets/" "photo-style.css")
  ;                (string-append photo-destination-dir "html/" "stylesheets/photo-style.css")
  ;                #t)

  (if (>= photo-show-verbosity-level 1) (display-message (string-append (as-string (length photo-list)) " " "photos pages generated")))
  (if (>= photo-show-verbosity-level 1) (display-message "DONE. Bring  index.html  up in you browser to start the show."))))

(define photo-list-length 0) ; assigned in make-photo-show

(define (photo-page-body-colors)
  (list
    'bgcolor default-photo-background-color 'text default-photo-text-color
    'link default-photo-text-color 'vlink default-photo-text-color))



; Make the photo index file. The parameter photo-list is a list of photo ASTs.
(define (make-photo-index photo-list previous-film-path next-film-path)
 (let* ((hs-space (horizontal-space 3))
        (overview-count (tabular-page-count tabular-overview-dimension (length photo-list)))
        (down-links (map (lambda (n) (string-append "html/" "index" "-" (as-string tabular-overview-dimension) "-" (as-string n) "." "html"))
                         (number-interval 1 overview-count)))
        (first-photo-url (if (not (null? photo-list))
                             (string-append "html/" (picture-name-to-url (photo-file-name-of (car photo-list)) "stop"))
                             ""))
        (last-photo-url (if (not (null? photo-list))
                             (string-append "html/" (picture-name-to-url (photo-file-name-of (last photo-list)) "stop"))
                             ""))
       )
  (write-html (list photo-show-html-mode 'prolog)
  (html
   (head (title (text-choice "Fotooversigt" "Photo index"))
         (script "" 'src (string-append "html/javascript/photoNavigate.js") 'type "text/javascript")
;         (link 'href "html/stylesheets/photo-style.css"  'rel "stylesheet" 'title "photo-style" 'type "text/css")
         (meta 'name "Generator" 'content "LAML")
   )
   (body  
    'onload (js-call "photoNavigate" 
                (append
                  (map string-it-single 
                          (list (if (and next-film-path (directory-exists? (string-append photo-destination-dir next-film-path)))
                                    (string-append next-film-path "index.html")
                                    "")
                                (if (and previous-film-path (directory-exists? (string-append photo-destination-dir previous-film-path)))
                                    (string-append previous-film-path "index.html")
                                    "")
                                ""
                                (if home-url home-url "")
                                first-photo-url  ; x
                                "" ; y
                                last-photo-url
                                ))
                   (list (js-string-array down-links))
                )
             )

    (photo-page-body-colors)
    (left-middle-right-banner 
          (con 
            (if (and previous-film-path (directory-exists? (string-append photo-destination-dir previous-film-path)))
                (a (text-choice "Forrige film" "Previous film") 'href (string-append previous-film-path "index.html") 'class "menu-item" (tool-tip "p"))
                (text-choice "Forrige film" "Previous film"))  hs-space
            (if (and next-film-path (directory-exists? (string-append photo-destination-dir next-film-path)))
                (a (text-choice "Nste film" "Next film") 'href (string-append next-film-path "index.html") 'class "menu-item" (tool-tip "n"))
                (text-choice "Nste film" "Next film"))

          )
          (em film-identification)
          (con
           (if home-url
               (a (text-choice "Hjem" "Home") 'href home-url  (tool-tip "u"))
               "") hs-space
           (a (text-choice "Om" "About") 'href (string-append "html/" (about-file-name) ".html") ) hs-space
           (a (text-choice "Hjlp" "Help") 'href (string-append "html/" (help-file-name) ".html") ))
    )
    (h1 (if photo-show-title photo-show-title (text-choice "Fotooversigt" "Photo Index")))
    (if (not (null? photo-list))
        (table 'border "0"
               (map2 photo-index-entry photo-list (number-interval 1 (length photo-list))))
        (em (text-choice "Ingen billeder" "No pictures")))

    (p)
;    (if last-url (a-tag last-url (text-choice "Fortsttelse" "Continuation of show")) "")
    (vertical-space 2)
    (font-size "2" photo-show-credit) (br)
    (font 'size "2" (when-generated))
   ))
   (string-append photo-destination-dir "index.html"))))

; Make an entry in the photo index
(define (photo-index-entry photo-entry n)
 (let ((caption (photo-lower-caption-of photo-entry))
       (timing (photo-show-time-of photo-entry)))
  (tr (td 'width "50" (as-string n) _ "." 'align "right" (horizontal-space 2))
      (td 'width "300" 
           (a 'href
             (string-append "html/" 
                            (cond ((eq? photo-show-index-linking 'stop) (picture-name-to-url (photo-file-name-of photo-entry) "stop"))
                                  ((eq? photo-show-index-linking 'run) (picture-name-to-url (photo-file-name-of photo-entry)))
                                  (else (laml-error "photo-index-entry:" "Unknown photo-show-index-linking" photo-show-index-linking))))
             (if caption caption (con (text-choice "Fotonummer" "Photo number") (horizontal-space 1) (as-string n)))
             ))
      (td 'width "50"
          (if (= 0 (remainder (- n 1) (* tabular-overview-dimension tabular-overview-dimension)))
              (tabular-link "html/" (+ 1 (quotient (- n 1) (* tabular-overview-dimension tabular-overview-dimension))) "underline")
              (horizontal-space 1)))
      (td 'width "200" (font-size 2 (photo-file-name-of photo-entry)))
      (td 'width "100" (font-size 2 (con (as-string timing) " " (text-choice "sekunder" "seconds"))))
  )))

; Return the number of tabular overview pages of dimension dim in a collection of photo-numbers photos.
(define (tabular-page-count dim photo-numbers)
  (+ (quotient (- photo-numbers 1) (* dim dim)) 1))

(define (tabular-url init-url-path dim i)
 (let ((url (string-append (tabular-page-name dim i) "." "html"))
      )
  (string-append init-url-path url)))

(define (tabular-link init-url-path i . optional-parameter-list)
 (let ((text-decoration (optional-parameter 1 optional-parameter-list "none"))
       (short-cut-letter (optional-parameter 2 optional-parameter-list #f))
       (times (string-append (as-string tabular-overview-dimension) " x " (as-string tabular-overview-dimension)))
      )
  (a 'href (tabular-url init-url-path tabular-overview-dimension i)  times 'css:text-decoration text-decoration (tool-tip short-cut-letter))))



; ---------------------------------------------------------------------------------------------------

; Take one level off relative-file-name. A generally useful function.
(define (take-one-level-from relative-file-name)
 (let ((forward-slash-pos (find-in-string relative-file-name #\/))
       (backward-slash-pos (find-in-string relative-file-name #\\))
       (lgt (string-length relative-file-name))
      )
   (cond ((and forward-slash-pos backward-slash-pos (<= forward-slash-pos backward-slash-pos))
             (substring relative-file-name (+ 1 forward-slash-pos) lgt))
         ((and forward-slash-pos backward-slash-pos (<= backward-slash-pos forward-slash-pos))
             (substring relative-file-name (+ 1 backward-slash-pos) lgt))
         (forward-slash-pos
             (substring relative-file-name (+ 1 forward-slash-pos) lgt))
         (backward-slash-pos
             (substring relative-file-name (+ 1 backward-slash-pos) lgt))
         (else #f))))



(define (photo-filename-leq? fn1 fn2)
  (string<=? (downcase-string fn1) (downcase-string fn1)))
  
;; Return a list of photo-ASTs sorted alphabetically, in the LAML startup directory.
;; A photo entry is a list of the form ("photo-file.ext" show-time "upper-caption" upper-size "lower-caption" lower-size)
;; upper-caption and/or lower-caption may be #f in case no caption apply; In that case upper-size and lower-size (integers, seconds)
;; do not matter.
(define (photo-asts-in-current-directory)
  (let* ((dir-list (directory-list (startup-directory)))
         (picture-list
           (filter 
             (lambda (fn) (or (equal? "jpg" (file-name-extension fn)) (equal? "gif" (file-name-extension fn))
                              (equal? "JPG" (file-name-extension fn)) (equal? "GIF" (file-name-extension fn))))
             dir-list))
         (sorted-picture-list (sort-list picture-list photo-filename-leq?))
        )
    (map make-default-photo-entry picture-list)))



(define (end-page)
  (html
   (head (title (text-choice "Slut side" "End page"))
         (script "" 'src (string-append "javascript/photoNavigate.js") 'type "text/javascript")
         (link 'href "stylesheets/photo-style.css"  'rel "stylesheet" 'title "photo-style" 'type "text/css")
         (meta 'name "Generator" 'content "LAML")
   )
   (body
    'onload (js-call "photoNavigate" 
                (append
                  (map string-it-single 
                          (list "" "" ""
                                "../index.html"
                                "" "" ""
                                ))
                  (list (js-string-array '()))
                )
             )

    (photo-page-body-colors)
    (center
       (vertical-space 4)
       (font 'size "6" (b (text-choice "Slut" "The End"))) (br)
       (vertical-space 2)
       (a 'href "../index.html" (font-size '2 (text-choice "Indhold" "Index")) 'css:text-decoration "none")  (br)
       (if home-url 
           (a 'href (string-append "../" home-url) (font-size 2 (text-choice "Hjem" "Home")) 'css:text-decoration "none")
           "")

    ))))

;; Return a sublist of photo-input-list, namely the photo entries in number-list.
;; The first photo is number 1.
(define (take-subset-by-numbers number-list photo-input-list)
   (if (null? number-list)
       '()
       (let ((actual-photo (get-photo-number (car number-list) photo-input-list)))
         (cons actual-photo (take-subset-by-numbers (cdr number-list) photo-input-list)))))
 
 (define (get-photo-number i phlst)
   (list-ref phlst (- i 1)))




; ------------------------------------------------------------------------------------------------------------------------

(define as-number-or-false
  (lambda (x)
    (cond ((or (string? x) (number? x)) (as-number x))
          ((and (boolean? x) (not x)) #f)
          (else (laml-error "as-number-or-false: expects number string or #f" x)))))

(define (photo-show-bridging! photo-show-ast)
 (let ((html-dir (string-append photo-destination-dir "html/"))
       (internal-dir (string-append photo-destination-dir "internal/"))
      )
  (display-message 
   (string-append 
     "The XML-in-LAML photo show generator, LAML " laml-version "."
   ))

  (if (>= photo-show-verbosity-level 1) (display-message "Processing " film-identification))

  (if (not (directory-exists? html-dir))
      (make-directory-in-directory photo-destination-dir "html/"))
  (if (not (directory-exists? internal-dir))
      (make-directory-in-directory photo-destination-dir "internal/"))
 
  (let* (
 
         (title-of-photo-show (aggregated-ast-cdata-contents (ast-subtree photo-show-ast "title-of-show")))
         (upper-caption-ast (ast-subtree photo-show-ast "upper-caption"))
         (lower-caption-ast (ast-subtree photo-show-ast "lower-caption"))
         (photo-show-upper-caption (aggregated-ast-cdata-contents upper-caption-ast))
         (photo-show-lower-caption (aggregated-ast-cdata-contents lower-caption-ast))
         (photo-show-upper-size (as-number (defaulted-get-prop 'size (ast-attributes upper-caption-ast) 3)))
         (photo-show-lower-size (as-number (defaulted-get-prop 'size (ast-attributes lower-caption-ast) 3)))
 
         (photo-list-ast (ast-subtree photo-show-ast "photo-list"))
         (photos-in-current-directory-ast (ast-subtree photo-show-ast "photos-in-current-directory"))
 
         (time-attr (as-number (get-prop 'time (ast-attributes photo-show-ast))))
         (background-color-attr (defaulted-get-prop 'background-color (ast-attributes photo-show-ast) (rgb-color-encoding 'black)))
         (text-color-attr (defaulted-get-prop 'text-color (ast-attributes photo-show-ast) (rgb-color-encoding 'white)))
         (home-path-attr (defaulted-get-prop 'home-path (ast-attributes photo-show-ast) #f))
         (photo-height-attr (as-number-or-false (defaulted-get-prop 'default-height (ast-attributes photo-show-ast) #f)))
         (photo-width-attr (as-number-or-false (defaulted-get-prop 'default-width (ast-attributes photo-show-ast) #f)))

         (previous-film-attr (defaulted-get-prop 'previous-film (ast-attributes photo-show-ast) #f))
         (next-film-attr (defaulted-get-prop 'next-film (ast-attributes photo-show-ast) #f))
         (before-first-attr (as-symbol (defaulted-get-prop 'before-first (ast-attributes photo-show-ast) "last-picture")))
         (after-last-attr (as-symbol (defaulted-get-prop 'after-last (ast-attributes photo-show-ast) "first-picture")))

         (transition-attr (defaulted-get-prop 'transition (ast-attributes photo-show-ast) "none"))
         (verbosity-level-attr (as-number (defaulted-get-prop 'verbosity-level (ast-attributes photo-show-ast) 2)))
         (langauge-attr (defaulted-get-prop 'language (ast-attributes photo-show-ast) "english"))
         (html-mode-attr (as-symbol (defaulted-get-prop 'html-mode (ast-attributes photo-show-ast) "raw")))
         (index-linking-attr (as-symbol (defaulted-get-prop 'index-linking (ast-attributes photo-show-ast) "stop")))
         (default-size-attr (as-symbol (defaulted-get-prop 'default-size (ast-attributes photo-show-ast) "auto")))
         (default-orientation-attr (as-symbol (defaulted-get-prop 'default-orientation (ast-attributes photo-show-ast) "landscape")))
         (tabular-overview-dimension-attr (as-number (defaulted-get-prop 'tabular-overview-dimension (ast-attributes photo-show-ast) "3")))

         (copy-film-attr (as-boolean (defaulted-get-prop 'copy-film (ast-attributes photo-show-ast) "false")))
         (copy-destination-attr (defaulted-get-prop 'copy-destination (ast-attributes photo-show-ast) "c:/temp/"))
         (landscape-percentage-attr (as-number (defaulted-get-prop 'landscape-percentage (ast-attributes photo-show-ast) default-landscape-percentage)))
         (portrait-percentage-attr (as-number (defaulted-get-prop 'portrait-percentage (ast-attributes photo-show-ast) default-portrait-percentage)))

         (camera-attr (defaulted-get-prop 'camera (ast-attributes photo-show-ast) #f))
        )

    (set! camera-used camera-attr)
    (set! photo-show-verbosity-level verbosity-level-attr)
    (set! default-upper-caption 
          (if (not (blank-string? photo-show-upper-caption))
              photo-show-upper-caption
              #f))
    (set! default-lower-caption 
          (if (not (blank-string? photo-show-lower-caption))
              photo-show-lower-caption
              #f))
    (set! default-upper-size photo-show-upper-size)
    (set! default-lower-size photo-show-lower-size)
    (set! default-time time-attr)
    (set! photo-show-title title-of-photo-show)
    (set! home-path home-path-attr)
    (set! home-url (if home-path (string-append home-path "index.html") #f))
    (set! default-photo-text-color text-color-attr)
    (set! default-photo-background-color background-color-attr)
    (set! language-preference (as-symbol langauge-attr))
    (set! default-transition transition-attr)
    (set! photo-show-html-mode html-mode-attr)
    (set! photo-show-index-linking index-linking-attr)

    (set! photo-show-previous-film previous-film-attr)
    (set! photo-show-next-film next-film-attr)
    (set! photo-show-before-first before-first-attr)
    (set! photo-show-after-last after-last-attr)
    
    (set! default-photo-size default-size-attr)
    (set! default-photo-orientation default-orientation-attr)
    (set! tabular-overview-dimension tabular-overview-dimension-attr)

    (set! landscape-percentage landscape-percentage-attr)
    (set! portrait-percentage portrait-percentage-attr)

    (set! photo-show-credit
          (text-choice 
           (span "LAML fotofremviseren er programmeret af Kurt Nrmark, Aalborg Universitet med brug af " 
                 (a 'href "http://www.cs.auc.dk/~normark/laml/" "LAML") "teknologi.")
 
           (span "The LAML photo show is programmed by Kurt Nrmark, Aalborg University by use of "
                 (a 'href "http://www.cs.auc.dk/~normark/laml/" "LAML") "technology.")))

    (cond (photo-list-ast 
            (let ((given-photo-ast-list (traverse-and-collect-all-from-ast photo-list-ast (ast-of-type? 'element-name "photo") id-1)))
              (set! current-photo-ast photo-show-ast)
              (set! photo-list-length (length given-photo-ast-list))     
              (make-photo-show given-photo-ast-list photo-show-previous-film photo-show-next-film) 
              (internal-save-photo-ast photo-show-ast)
              (internal-save-film-description-entry film-identification photo-show-title photo-list-length)  
              (make-photo-overview-show given-photo-ast-list tabular-overview-dimension)
              (make-laml-fragment-file given-photo-ast-list)

              (if copy-film-attr
                  (copy-pictures! given-photo-ast-list (ensure-final-character copy-destination-attr #\/)))

            )
          )
 
          (photos-in-current-directory-ast
            (let* ((derived-photo-ast-list-0 (photo-asts-in-current-directory))
                   (derived-photo-ast-list 
                     (if (and camera-used (equal? camera-used "Canon-G3"))
                         (let ((rotated-photo-names (filter rotated-photo-name? (map photo-file-name-of derived-photo-ast-list-0))))
                           (remove-redundant-landscape-photos derived-photo-ast-list-0 rotated-photo-names))
                         derived-photo-ast-list-0))
                   (modified-ast
                     (make-ast 
                      (ast-element-name photo-show-ast)
                      (list (ast-subtree photo-show-ast "upper-caption")
                            (ast-subtree photo-show-ast "lower-caption")
                            (make-ast 
                              "photo-list"
                              derived-photo-ast-list
                              '()
                              'double
                              "laml-photo"))
                      (ast-attributes photo-show-ast)
                      'double
                      "laml-photo"))
                  )
              (set! current-photo-ast modified-ast)
              (set! photo-list-length (length derived-photo-ast-list))
              (make-photo-show derived-photo-ast-list photo-show-previous-film photo-show-next-film)
              (internal-save-photo-ast modified-ast)
              (internal-save-film-description-entry film-identification photo-show-title photo-list-length)

              (make-photo-overview-show derived-photo-ast-list tabular-overview-dimension)
              (make-laml-fragment-file derived-photo-ast-list)

              (if copy-film-attr
                  (copy-pictures! derived-photo-ast-list (ensure-final-character copy-destination-attr #\/)))

              (make-internal-photo-list! derived-photo-ast-list)
            )
          )
 
          (else (laml-error "photo-show-bridging!" "Either photo-list or photos-in-current-directory must be present in a photo-show")))

    (if home-path
        (let ((home-abs-path (string-append photo-destination-dir home-path)))
          (if (safe-to-write-home-photo-page? home-abs-path)
              (make-home-photo-index)
              (display-message "** No index.html file is written in " home-abs-path ". " 
                               "Reason: index.html is in danger of being overwritten. **"))))


    (make-help-page) (make-about-page)

    (write-html (list photo-show-html-mode 'prolog) (end-page) (string-append photo-destination-dir "html/" "end-page.html"))
    (write-html (list photo-show-html-mode 'prolog) (end-page) (string-append photo-destination-dir "html/" "end-page-stop.html"))

    ; Copy process-all script to photo home dir
    (let ((source-path (string-append photo-show-dir "scripts/process-all-films.laml"))
          (destination-path (string-append photo-destination-dir home-path "process-all-films.laml")))
      (if (not (file-exists? destination-path))
          (copy-file source-path destination-path)))

  )
 )
)

;; Is it considered safe to overwrite index.html in home-dir
(define (safe-to-write-home-photo-page? home-dir)
  (if (file-exists? (string-append home-dir "index.html"))
      (let ((index-html-text (read-text-file (string-append home-dir "index.html"))))
        (or 
           (substring-index index-html-text 0 "LAML Foto Hjemmeside")
           (substring-index index-html-text 0 "LAML Photo Home page")))
      #t))


; Canon G3 specific
(define (remove-redundant-landscape-photos photo-ast-list rotated-photo-names)
 (let ((downcased-rotated-photo-names (map downcase-string rotated-photo-names)))
  (filter 
   (lambda (photo-ast)
     (let* ((photo-name (photo-file-name-of photo-ast))
            (photo-name-proper (file-name-proper photo-name))
            (photo-name-ext    (file-name-extension photo-name))
          )
       (not (member (downcase-string (string-append photo-name-proper rotated-suffix-name "." photo-name-ext)) downcased-rotated-photo-names))))
   photo-ast-list)))


(define (internal-save-photo-ast ast)
 (let ((internal-dir (string-append photo-destination-dir "internal/")))
  (file-write ast (string-append internal-dir "photo-ast.lsp"))))

(define (internal-save-film-description-entry film-identification photo-show-title photo-list-length)
 (let ((internal-dir (string-append photo-destination-dir "internal/")))
  (file-write (list film-identification photo-show-title photo-list-length tabular-overview-dimension) (string-append internal-dir "film-description-entry.lsp")))) 


(define (first-url-of-photo-ast photo-ast film-path)
 (if photo-ast
     (let* ((photos (ast-subtrees (ast-subtree photo-ast "photo-list"))))
       (if (not (null? photos))
           (let* ((first-photo (car photos))
                  (jpg-file-name (get-prop 'file (ast-attributes first-photo)))
                  (file-name (file-name-proper jpg-file-name))
                 )
             (string-append "../" film-path "html/" file-name "." "html")) 
           ""))
     ""))


(define (last-url-of-photo-ast photo-ast film-path)
 (if photo-ast
     (let* ((photos (ast-subtrees (ast-subtree photo-ast "photo-list"))))
       (if (not (null? photos))
           (let* ((last-photo (last photos))
                  (jpg-file-name (get-prop 'file (ast-attributes last-photo)))
                  (file-name (file-name-proper jpg-file-name))
                 )
             (string-append "../" film-path "html/" file-name "." "html")) 
           ""))
     ""))

(define (first-tabular-url-of-photo-ast photo-ast film-path)
 (if photo-ast
     (let* ((photos (ast-subtrees (ast-subtree photo-ast "photo-list")))
            (tab-dim (as-number (defaulted-get-prop 'tabular-overview-dimension (ast-attributes photo-ast) "3")))
           )
       (if (not (null? photos))
           (string-append "../" film-path "html/" (tabular-page-name tab-dim 1) "." "html")
           ""))
     ""))

(define (last-tabular-url-of-photo-ast photo-ast film-path)
 (if photo-ast
     (let* ((photos (ast-subtrees (ast-subtree photo-ast "photo-list")))
            (photo-numbers (length photos))
            (tab-dim (as-number (defaulted-get-prop 'tabular-overview-dimension (ast-attributes photo-ast) "3")))
            (last-tab-page-number (tabular-page-count tab-dim photo-numbers))
           )
       (if (not (null? photos))
           (string-append "../" film-path "html/" (tabular-page-name tab-dim last-tab-page-number) "." "html")
           ""))
     ""))

(define (get-photo-ast path)
 (let ((ast-file-path (string-append (startup-directory) path "internal/" "photo-ast.lsp")))
   (if (file-exists? ast-file-path)
       (file-read ast-file-path)
       #f)))


; Make a size x size overview show.
(define (make-photo-overview-show photo-ast-list size)
 (let ((page-count
        (lambda (n m)
          (if (= (remainder n m) 0)
              (quotient n m)
              (+ (quotient n m) 1))))
       (overview-page-name 
         (lambda (n)
           (string-append "index" "-" (as-string size) "-" (as-string n))))
      )
   (let* ((pictures-per-page (* size size))
          (photo-lgt (length photo-ast-list))
          (photo-overview-page-count (page-count photo-lgt pictures-per-page))
          (photo-ast-list-list (sublist-by-rows pictures-per-page photo-ast-list))
          (photo-page-name-list (map overview-page-name (number-interval 1 photo-overview-page-count))))
    (map5
     (make-overview-photo-page size (+ (quotient (- (length photo-ast-list) 1) (* tabular-overview-dimension tabular-overview-dimension)) 1))
     photo-ast-list-list
     photo-page-name-list
     (if (not (null? photo-page-name-list)) (append (cdr photo-page-name-list) (list #f)) '())
     (if (not (null? photo-page-name-list)) (append (list #f) (butlast photo-page-name-list)) '())
     (number-interval 1 (length photo-page-name-list))
     ))))

(define (make-overview-photo-page size total-number)
 (let ((cur-map (curry-generalized map))
       (hs-space (horizontal-space 3))
      )
  (lambda (photo-list page-name next-page-name prev-page-name n)
   (write-html (list photo-show-html-mode 'prolog)
    (let* ((colors-list (list default-photo-background-color default-photo-text-color default-photo-text-color default-photo-text-color))
           (bg-color (first colors-list))
           (text-color (second colors-list))
           (link-color (third colors-list))
           (vlink-color (fourth colors-list))
           (down-links (map (lambda (pe) (string-append (file-name-proper (photo-file-name-of pe)) "-stop" "." "html")) photo-list))
           (last-tab-url-of-prev-film (if photo-show-previous-film (last-tabular-url-of-photo-ast (get-photo-ast photo-show-previous-film) photo-show-previous-film) ""))
           (first-tab-url-of-next-film (if photo-show-next-film (first-tabular-url-of-photo-ast (get-photo-ast photo-show-next-film) photo-show-next-film) ""))
          )
    (html 
     (head 
      (title (text-choice "LAML foto fremvisning" "LAML photo show"))
      (script "" 'src (string-append "javascript/photoNavigate.js") 'type "text/javascript")
      (link 'href "stylesheets/photo-style.css"  'rel "stylesheet" 'title "photo-style" 'type "text/css")
      (meta 'name "Generator" 'content "LAML")

     )
     (body 
      'onload (js-call "photoNavigate" 
                  (append 
                    (map string-it-single 
                          (list (if next-page-name (string-append next-page-name "." "html") first-tab-url-of-next-film)
                                (if prev-page-name (string-append prev-page-name "." "html") last-tab-url-of-prev-film)
                                ""
                                "../index.html"
                                "" "" ""
                                ))
                   (list (js-string-array down-links))
                  )
               )
      'bgcolor bg-color
      'text text-color
      'link link-color
      'vlink vlink-color

      (left-middle-right-banner 
       (con 
        (a (text-choice "Indhold" "Index") 'href "../index.html" 'class "menu-item" (tool-tip "u")) hs-space
        (cond (prev-page-name
                   (a (text-choice "Foregende" "Previous") 'href (string-append prev-page-name ".html") 'class "menu-item" (tool-tip "p")))
              ((not (empty-string? last-tab-url-of-prev-film))
                   (a (text-choice "Foregende" "Previous") 'href last-tab-url-of-prev-film 'class "menu-item" (tool-tip "p")))
              (else (text-choice "Foregende" "Previous")))
        hs-space
        (cond (next-page-name 
                 (a (text-choice "Nste" "Next") 'href (string-append next-page-name ".html") 'class "menu-item" (tool-tip "n")))
              ((not (empty-string? first-tab-url-of-next-film))
                 (a (text-choice "Nste" "Next") 'href first-tab-url-of-next-film 'class "menu-item" (tool-tip "n")))
              (else (text-choice "Nste" "Next")))
       )
 
       (span 'class "menu-item"  film-identification hs-space "-" hs-space (as-string n) ":" (as-string total-number) )

       (con
        (if home-url
            (a (text-choice "Hjem" "Home") 'href (string-append "../" home-url) 'class "menu-item")
            "") hs-space
        (a (text-choice "Om" "About") 'href (string-append (about-file-name) ".html") 'class "menu-item") hs-space
        (a (text-choice "Hjlp" "Help") 'href (string-append (help-file-name) ".html") 'class "menu-item")
        )
       )

      (center (h1 photo-show-title)) (p)

      (center
       (table 'border "0"  'cellpadding "8mm"
         (map (compose tr (cur-map (image-in-index size))) (sublist-by-rows size photo-list)))
      )        

    )
   ))
   (string-append photo-destination-dir "html/" page-name ".html")))))

(define (image-in-index size)
 (let ((percent-number (lambda (n) (string-append (as-string n) "%"))))
  (lambda (this-photo-entry)
   (let* ((this-file-name (photo-file-name-of this-photo-entry))
          (this-file-name-proper (file-name-proper this-file-name))
          (this-path (photo-path-of this-photo-entry))
          (this-upper-caption (photo-upper-caption-of this-photo-entry))
          (this-upper-size (photo-upper-size-of this-photo-entry))
          (this-lower-caption (photo-lower-caption-of this-photo-entry))
          (this-photo-orientation (photo-orientation-of this-photo-entry))

          (real-picture-url-prefix 
           (string-append picture-url-prefix 
                          (if this-path this-path "")))
          )
     (td (center 
          (a 'href (string-append this-file-name-proper "-stop" "." "html")
            (if (eq? this-photo-orientation 'landscape)
                (img 'border "0" 'width (as-string (quotient photo-table-window-width size)) 'src (string-append real-picture-url-prefix this-file-name) 'alt "")
                (img 'border "0" 'height (as-string (quotient photo-table-window-height size)) 'src (string-append real-picture-url-prefix this-file-name) 'alt "")
            )
          ))
         (br)
         (center (a 'href (string-append "laml-fragments.html" "#" this-file-name-proper ) (font 'size "2" this-file-name-proper) 'css:text-decoration "none"))
     )
   )
  )
 )
)



(define (make-help-page)
 (let ((lis (lambda x (li x 'css:margin-bottom "7pt")))
       (kn-url "http://www.cs.auc.dk/~normark/")
       (hs-space (horizontal-space 3))
      )
 (write-html (list photo-show-html-mode 'prolog)
    (let* ((colors-list (list default-photo-background-color default-photo-text-color default-photo-text-color default-photo-text-color))
           (bg-color (first colors-list))
           (text-color (second colors-list))
           (link-color (third colors-list))
           (vlink-color (fourth colors-list)))
    (html 
     (head 
      (title "Hjlp om LAML foto fremvisning")
      (meta 'name "Generator" 'content "LAML")

     )
     (body 

       (left-middle-right-banner 
        (a 'href "../index.html" (font-size '2 (text-choice "Indhold" "Index")) )
        ""
        (con 
          (a (text-choice "Om" "About") 'href (string-append (about-file-name) ".html") 'class "menu-item") hs-space
          (if home-url
           (a (text-choice "Hjem" "Home") 'href (string-append "../" home-url))
            "")
        )
       )

      'bgcolor bg-color
      'text text-color
      'link link-color
      'vlink vlink-color

      (h1 "Hjlp til LAML foto fremvisning")

      (div
         (p "Denne fotofremvisning er genereret af et program, som kaldes 'LAML fotogeneratoren'.
          Kort fortalt vil dette program generere en mngde HTML filer, som tillader dig at 
          se dine fotos i enhver Internet browser.")

         (p "Hjemmesiden giver et overblik og alle
             fotosider, som er processeret med LAML fotogeneratoren.")

         (p "Film indholdssiden giver et overblik over alle billeder i en enkelt 'film'. Dette inkluderer links til de enkelt billeder
             og 'n x n' links to oversigtssider med typisk 4, 9 eller 16 biller.")

         (p "En n x n side med typisk 4, 9 eller 16 viser en tabel af billeder, som er nyttigt for at danne sig en oversigt over 
             billederne i en film.")

         (p "Vi vil her beskrive, hvordan du kan navigere i dine billeder")

         (ul 
           (lis "Fra foto hjemmesiden man vlge en vilkrlig film. Tasterne 1-9 og a-m vlger en af filmene fra toppen af listen.
                 'x', 'y' og 'z' vlger de tre film fra bunden af listen.
                ")
           (lis "Fra en film indholdsside kan man direkte vlge et vilkrligt billede med musen. Man kan ogs vlge en 
                 n x n oversigt. Tasterne 1-9 og a-m vlger en n x n side. Tasten 'x' vlger det frste billede,
                 og 'z' vlger det sidste billede.
                 Film indholdssiderne kan vre kdet sammen med links, som kan navigeres med 'n' og 'p'.
                 Tasten 'u' naviger til foto hjemmesiden.
                 ")
           (lis "Fra en n x n oversigtsside kan man vlge en af billederne for visning i fuld strrelse.
                 Man kan ogs navigere frem og tilbage mellem oversigtsider, samt op til fil indholdssiden.
                 Dette gres via tastaturet med hhv. 'n', 'p', og 'u'.
                 Hvis filmene er forbundet, kan man endvidere skifte til nste og foregende film fra hhv. 
                 sidste og frste n x n oversigsside (igen med 'n' og 'p').
                 Hvis man vlger navnet p et billede bringes man til en side med LAML foto fragmenter, som
                 er nyttig hvis man nsker at billedet skal indg i et andet 'album'.")
           (lis "Fra en billedside uden automatisk skift kan man navigere til nste og foregende
                 side samt til film indholdssiden og fot hjemmesiden'. Tasterne 'n' og 'p' gr frem og tilbage.
                 Tasten 'u' navigerer til den relevante n x n side. Tasten 'x' navigerer til foto indholdssiden.
                 Man kan ogs vlge at igangstte automatisk skift fra
                 side til side med 'Kr', eller via tasten 'r'. 
                 Hvis man vlger navnet p et billede (i allerverste linie p skrmen) bringes man til en side med LAML foto fragmenter, som
                 er nyttig hvis man nsker at billedet skal indg i et andet 'album'.
                 ")
           (lis "Fra en billedside med automatisk skift kan man vlge at stoppe med 'Stop' samt at g til indholdssiden
                 eller hjemmesiden. Tasten 's' betyder 'stop'.")
         )

         (p "Flgende giver en oversigt over tasturgenveje fra billedsiderne uden automatisk skift:")
 
         (indent-pixels 10         
          (table 'border "1"
           (tr (td "Nste") (td "n, RETUR"))
           (tr (td "Foregende") (td "p"))
           (tr (td "n x n sider") (td "u"))
           (tr (td "Film indholdssiden") (td "x"))
           (tr (td "Kr") (td "r, MELLEMRUM"))
           (tr (td "Stop") (td "s, MELLEMRUM"))))

         (p "Bemrk at disse tastaturgenveje ogs vises, nr musen hviler over et link.")

         (p "Hvis du nsker at generere en LAML fotofremvisning kan du lse om dette p
             en separat" (a 'href "http://www.cs.auc.dk/~normark/scheme/styles/xml-in-laml/photo-show/man/photo-show.html" "manual side") _ "."
            "Hvis du nsker at vide mere om LAML og LAML software er du velkommen til at se p" 
            (a 'href "http://www.cs.auc.dk/~normark/laml/" "LAML hjemmesiden") _ "."
         )

         (vertical-space 1)

         (p 
           "Kurt Nrmark" (br)
           (a 'href kn-url kn-url) (br)
           "normark@cs.auc.dk")
        )

    )
   ))
   (string-append photo-destination-dir "html/" "help-dk" ".html"))

   (write-html (list photo-show-html-mode 'prolog)
    (let* ((colors-list (list default-photo-background-color default-photo-text-color default-photo-text-color default-photo-text-color))
           (bg-color (first colors-list))
           (text-color (second colors-list))
           (link-color (third colors-list))
           (vlink-color (fourth colors-list)))
    (html 
     (head 
      (title "Help about LAML photo show")
      (meta 'name "Generator" 'content "LAML")

     )
     (body 

       (left-middle-right-banner 
        (a 'href "../index.html" (font-size '2 (text-choice "Indhold" "Index")) )
        ""
        (con 
          (a (text-choice "Om" "About") 'href (string-append (about-file-name) ".html") 'class "menu-item") hs-space
          (if home-url
           (a (text-choice "Hjem" "Home") 'href (string-append "../" home-url))
            "")
        )
       )

      'bgcolor bg-color
      'text text-color
      'link link-color
      'vlink vlink-color

      (h1 "Help about LAML photo shows")

      (div "In the current version the 'Help Page' is only available" (a 'href "help-dk.html" "in Danish") _ ".")

    )
   ))
   (string-append photo-destination-dir "html/" "help-en" ".html"))

))


(define (make-about-page)
 (let ((lis (lambda x (li x 'css:margin-bottom "7pt")))
       (kn-url "http://www.cs.auc.dk/~normark/")
       (hs-space (horizontal-space 3))
      )
 (write-html (list photo-show-html-mode 'prolog)
    (let* ((colors-list (list default-photo-background-color default-photo-text-color default-photo-text-color default-photo-text-color))
           (bg-color (first colors-list))
           (text-color (second colors-list))
           (link-color (third colors-list))
           (vlink-color (fourth colors-list)))
    (html 
     (head 
      (title "Om LAML fotofremvisning")
      (meta 'name "Generator" 'content "LAML")

     )
     (body 

       (left-middle-right-banner 
        (a 'href "../index.html" (font-size '2 (text-choice "Indhold" "Index")) )
        ""
        (con 
          (a (text-choice "Hjlp" "Help") 'href (string-append (help-file-name) ".html") 'class "menu-item")
          hs-space
          (if home-url (a (text-choice "Hjem" "Home") 'href (string-append "../" home-url)) "")
        )
       )

      'bgcolor bg-color
      'text text-color
      'link link-color
      'vlink vlink-color

      (h1 "Om LAML foto fremvisning")

      (div

         (p "Dette foto show er produceret af et program kaldet" (b "LAML fotogeneratoren") _ "," "som er designet og programmeret af Kurt Nrmark,
             Aalborg Universitet. Programmet genererer en rkke HTML filer omkring en mngde foto filer. Hermed kan fotografierne vises bekvemt i
             enhver Internet browser.")

         (p "Beskrivelsen af fremvisningens egenskaber er lavet i XML, mere prcist i et format der kaldes" 
            (a 'href "http://www.cs.auc.dk/~normark/scheme/tutorial/xml-in-laml/xml-in-laml.html" "XML-in-LAML") _ ".")

         (p (a 'href "http://www.w3.org/XML/" "XML") "er en standard for beskrivelse af strukturerede data.")

         (p (a 'href "http://www.cs.auc.dk/~normark/laml/" "LAML") "er en softwarepakke som understtter XML og HTML i programmeringssproget" 
             (a 'href "http://www.schemers.org" "Scheme") "(et sprog i Lisp familien).
             Mere specifikt understtter LAML muligheden for at 
             generere et" (em "spejl") "af et vilkrlig XML sprog for hvilket der findes en Document Type Definition (DTD).
             LAML har vret anvendt i mange forskellige sammenhnge, primrt inden for produktion af forskellige" 
             (a 'href "http://www.cs.auc.dk/~normark/leno/" "undervisningsmaterialer") "og
             " (a 'href "http://www.cs.auc.dk/~normark/elucidative-programming/index.html" "programdokumentationsvrktj") _ "." "
             LAML har vret under udvikling siden 1999.")

         (p "LAML er frit software tilgngelig fra LAML hjemmesiden."
            "LAML beror p en Scheme processor (typisk, men ikke ndvendigvis" 
            (a 'href "http://www.plt-scheme.org/" "PLT")_"'s"
            (a 'href "http://www.plt-scheme.org/software/mzscheme/" "MzScheme") "). Det anbefales, at processeringen af et LAML dokument aktiveres fra"
            (a 'href "http://www.gnu.org/software/emacs/emacs.html" "Emacs") _ "." "De fleste 
             Scheme systemer samt Emacs er ligeledes frit software." 
         )

         (p "Hvis du nsker at generere en LAML fotofremvisning kan du lse om dette p
             en separat" (a 'href "http://www.cs.auc.dk/~normark/scheme/styles/xml-in-laml/photo-show/man/photo-show.html" "manual side") _ "."
         )

         (vertical-space 1)

         (p 
           "Kurt Nrmark" (br)
           (a 'href kn-url kn-url) (br)
           "normark@cs.auc.dk")
           
      )

    )
   ))
   (string-append photo-destination-dir "html/" "about-dk" ".html"))

   (write-html (list photo-show-html-mode 'prolog)
    (let* ((colors-list (list default-photo-background-color default-photo-text-color default-photo-text-color default-photo-text-color))
           (bg-color (first colors-list))
           (text-color (second colors-list))
           (link-color (third colors-list))
           (vlink-color (fourth colors-list)))
    (html 
     (head 
      (title "About LAML photo show")
      (meta 'name "Generator" 'content "LAML")

     )
     (body 

       (left-middle-right-banner 
        (a 'href "../index.html" (font-size '2 (text-choice "Indhold" "Index")) )
        ""
        (con 
          (a (text-choice "Hjlp" "Help") 'href (string-append (help-file-name) ".html") 'class "menu-item")
          hs-space
          (if home-url (a (text-choice "Hjem" "Home") 'href (string-append "../" home-url)) "")
        )
       )

      'bgcolor bg-color
      'text text-color
      'link link-color
      'vlink vlink-color

      (h1  "About LAML photo shows")

      (div "In the current version the 'about page' is only available" (a 'href "about-dk" "in Danish") _ ".")

      

    )
   ))
   (string-append photo-destination-dir "html/" "about-en" ".html"))



))



   
; Top level photo home page.
(define (make-home-photo-index)
 (let* ((film-path (lambda (foto-entry) (string-append (film-path-of foto-entry) "/" "index.html")))

        (hs-space (horizontal-space 3))
        (film-list (photo-film-list (string-append photo-destination-dir home-path)))
        (film-list-1 (filter (lambda (film-entry) (numeric-string-component (film-name-of film-entry))) film-list))
        (film-list-2 (filter (lambda (film-entry) (not (numeric-string-component (film-name-of film-entry)))) film-list))
        (sorted-film-list-1 (sort-list film-list-1 film-numeric-leq?))
        (sorted-film-list-2 (sort-list film-list-2 film-alphabetic-leq?))
        (total-film-list (append sorted-film-list-2 sorted-film-list-1))
        (film-list-lgt (length total-film-list)) 
        (reverse-sorted-film-list (reverse total-film-list))
        (first-film (if (null? total-film-list) #f (car total-film-list)))
        (first-film-path (if first-film (film-path-of first-film) #f))
        (photo-count (sum-list (map film-size-of total-film-list)))

        (down-links (map film-path total-film-list))
       )

  (write-html (list photo-show-html-mode 'prolog)
    (html
     (head (title (text-choice "LAML Foto Hjemmeside" "LAML Photo Home page"))
           (if first-film-path (script "" 'src (string-append first-film-path "/" "html/javascript/photoNavigate.js") 'type "text/javascript") '())
           (meta 'name "Generator" 'content "LAML")
;           (if first-film-path 
;               (link 'href (string-append first-film-path "/" "html/stylesheets/photo-style.css")  'rel "stylesheet" 'title "photo-style" 'type "text/css")
;               '())
     )
     (body  
      'onload (js-call "photoNavigate" 
                  (append
                    (map string-it-single 
                          (list "" "" "" ""   
                                (if (>= film-list-lgt 3) (film-path (third reverse-sorted-film-list)) "")  ; x
                                (if (>= film-list-lgt 2) (film-path (second reverse-sorted-film-list)) "") ; y
                                (if (>= film-list-lgt 1) (film-path (first reverse-sorted-film-list)) "")  ; z
                                ))
                    (list (js-string-array down-links)))
              )

      (left-middle-right-banner 
        ""
        "" 

        (con
         (a (text-choice "Om" "About") 'href (string-append first-film-path "/" "html/" (about-file-name) ".html") 'class "menu-item") hs-space
         (a (text-choice "Hjlp" "Help") 'href (string-append first-film-path "/" "html/" (help-file-name) ".html") 'class "menu-item")
        )
     )

      (photo-page-body-colors)
   
      (h1 (text-choice "Foto Hjemmeside" "Photo Page"))
      (table 'border "0" (map photo-home-index-entry total-film-list))
      (p)
      (font-size 2 
       (text-choice (span "Ialt" (as-string photo-count) "fotos.")
                    (span "In total" (as-string photo-count) "photos.")))
      (p)
      (font-size "2" photo-show-credit) (br)
      (font 'size "2" (when-generated))
     ))
   (string-append photo-destination-dir home-path "index.html"))))


; film description entry selectors
(define film-path-of (make-selector-function 1 "film-path-of"))
(define film-name-of (make-selector-function 2 "film-name-of"))
(define film-title-of (make-selector-function 3 "film-title-of"))
(define film-size-of (make-selector-function 4 "film-size-of"))
(define film-tabular-overview-dimension (make-selector-function 5 "film-tabular-overview-dimension"))

(define (photo-home-index-entry film-entry)
  (li (a 'href (string-append (film-path-of film-entry) "/" "index.html") (film-name-of film-entry))
      "-" (film-title-of film-entry)
      "(" _ (as-string (film-size-of film-entry)) (text-choice "billeder" "pictures") _ ")"))

(define (photo-home-index-entry film-entry)
  (tr 
    (td 'width "20" (horizontal-space 1))
    (td 'width "100" (a 'href (string-append (film-path-of film-entry) "/" "index.html") (capitalize-string-nd (film-name-of film-entry))))
    (td 'width "300" (film-title-of film-entry))
    (td 'width "100" (as-string (film-size-of film-entry)) (text-choice "billeder" "pictures"))
  )
)
        
(define (film-numeric-leq? fde1 fde2)
  (<= (numeric-string-component (film-name-of fde1)) (numeric-string-component (film-name-of fde2))))

(define (film-alphabetic-leq? fde1 fde2)
  (string<=? (film-name-of fde1) (film-name-of fde2)))


; Return a list of film description entries (film-name film-title)
; captured from home-dir
(define (photo-film-list home-dir)
 (let* ((subdir-list (directory-list home-dir))
        (photo-dirs (filter (lambda (subdir) (is-laml-photo-dir? home-dir subdir)) subdir-list))
        (non-photo-dirs (filter (lambda (subdir) (not (is-laml-photo-dir? home-dir subdir))) subdir-list))
       )
   (map (lambda (subdir) (cons subdir (read-film-description-entry home-dir subdir)))
        photo-dirs)))

(define (is-laml-photo-dir? path dir)
 (let ((full-path (string-append path dir "/")))
  (file-exists? (string-append full-path "internal/" "film-description-entry.lsp"))))

(define (read-film-description-entry path dir)
  (file-read (string-append path dir "/" "internal" "/" "film-description-entry.lsp")))


; Create a HTML file with LAML photo fragment - ready to be copied.
(define (make-laml-fragment-file photo-ast-list)
   (write-html (list photo-show-html-mode 'prolog)
    (html
     (head (title (text-choice "LAML Fragmenter" "LAML Fragments"))
     )
     (body  

      (h1 (text-choice "LAML Fragmenter" "LAML Fragments"))

      (map laml-single-photo-fragment photo-ast-list)

      (vertical-space 30)
     
     ))
   (string-append photo-destination-dir "html/" "laml-fragments.html")))


(define (laml-single-photo-fragment photo-ast)
 (let* ((file-name (photo-file-name-of photo-ast))
        (proper-file-name (file-name-proper file-name)))
   (p (a 'name proper-file-name)
       "(photo 'time \"3\" 'path \"../"_ film-identification _  "/\""  " 'file " (string-it file-name) " 'size \"auto\" "
       "(upper-caption 'size \"6\" \"\")" ")" (horizontal-space 3)
       (a 'href (string-append proper-file-name "-" "stop" "." "html") (text-choice "Billede" "Picture")))))

(define (tool-tip letter)
  (if letter
      (list 'title 
            (string-append 
             (text-choice "Tastatur genvej: " "Keyboard shortcut: ")
             (as-string letter)))
      '()))

; ---------------------------------------------------------------------------------------------------

; Return first encountered number component of str.
; Return #f if no ciffer is encountered
(define (numeric-string-component str)
  (numeric-string-component-1 str 0 #f))

(define (as-natural-number ch)
  (- (as-number ch) (as-number #\0)))

(define (ciffer? ch)
  (turn-into-boolean (memv ch (list #\0 #\1 #\2 #\3 #\4 #\5 #\6 #\7 #\8 #\9 ))))

(define (numeric-string-component-1 str i res)
  (cond ((>= i (string-length str)) res)
        ((and (not res) (ciffer? (string-ref str i)))  (numeric-string-component-1 str (+ i 1) (as-natural-number (string-ref str i))))
        ((and res (ciffer? (string-ref str i)))  (numeric-string-component-1 str (+ i 1) (+ (* 10 res) (as-natural-number (string-ref str i)))))
        ((and res (not (ciffer? (string-ref str i)))) res)
        ((and (not res) (not (ciffer? (string-ref str i)))) (numeric-string-component-1 str (+ i 1) #f))
        (else (laml-error numeric-string-component str) "Internal error. Should not happen.")))
            
; --------------------------------------------------------------------------------------------------- 
; Photo copy support.

(define (copy-pictures! photo-ast-list absolute-path)
  (for-each
    (lambda (photo-ast)
      (let* ((photo-path (photo-path-of photo-ast))
             (photo-name (photo-file-name-of photo-ast))
             (source-path (string-append photo-destination-dir (if photo-path photo-path "") photo-name))
             (destination-path (string-append absolute-path (ensure-non-existing-file-in-dir photo-name absolute-path)))  ; (string-append absolute-path photo-name) 
            )
       (if (not (file-exists? destination-path))
           (begin
             (copy-file source-path destination-path)
             (if (>= photo-show-verbosity-level 1) 
                 (display-message "Copying" photo-name "to" destination-path))
           )
           (display-message "Destination file exists" destination-path "- NO copy made."))))
    photo-ast-list))


; ---------------------------------------------------------------------------------------------------
; Making internal photo-list -ready to put into LAML source.

(define (make-internal-photo-list! photo-ast-list)
  (write-text-file
    (string-append
      "  " "(photo-list" CR

      (accumulate-right string-append ""
        (map make-photo-clause-string photo-ast-list))

      CR "  " ")"
    )

    (string-append photo-destination-dir "/" "internal" "/" "photo-list")
  )
)

(define (make-photo-clause-string photo-ast)
 (let ((tm (photo-show-time-of photo-ast))
       (pth (photo-path-of photo-ast))
       (fl (photo-file-name-of photo-ast))
       (sz (photo-size-of photo-ast))
       (ori (photo-orientation-of photo-ast))
       (ls (photo-lower-size-of photo-ast))
       (us (photo-upper-size-of photo-ast))
       (uc (photo-upper-caption-of photo-ast))
       (lc (photo-lower-caption-of photo-ast))
      )

  (string-append
     "    " "(photo "
     (if tm (string-append "'time " (string-it (as-string tm)) " ") "")
     (if pth (string-append "'path " (string-it (as-string pth)) " ") "")
     (if fl (string-append "'file " (string-it (as-string fl)) " ") "")
     (if sz (string-append "'size " (string-it (as-string sz)) " ") "")
     (if ori (string-append "'orientation " (string-it (as-string ori)) " ") "")
     (if uc (string-append "(upper-caption " (if us (string-append "'size " (string-it (as-string us))) "") " " (string-it uc) ")") "") 
     (if lc (string-append "(lower-caption " (if ls (string-append "'size " (string-it (as-string ls))) "") " " (string-it lc) ")") "")
     ")" CR
  )
 )
)

; --------------------------------------------------------------------------------------------------- 
  

